package com.ae7q.communication;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.logging.*;
import java.util.prefs.*;
import javax.swing.*;
//import javax.swing.event.*;

import com.ultimeth.util.*;

//==================================================================================================
/** AgwpeTerminal is an AX.25 packet terminal program for AGWPE.
 *
 *  @version    2006.04.09
 *  @author     Copyright © 2006 by Dean K. Gibson.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
@SuppressWarnings( "serial" )
class AgwpeTerminal extends Box {
    private static final String        TITLE            = "AX.25 packet terminal for AGWPE";
    private static final String        COPYRIGHT        = "Copyright \u00A9 2005 by Dean K. Gibson";
    private static final char          endOfLine        = '\r';
    private static final JFrame        jFrame           = new JFrame( TITLE );
    private static final JTabbedPane   jTabbedPane      = new JTabbedPane();
    private static final Map <String, AgwpeTerminal>
                                       hashMap          = new HashMap <String, AgwpeTerminal>();
    private static final AbstractList <Component>
                                       uiComponents     = new ArrayList <Component>();
    private static final Preferences   preferences      = Preferences
                                                          .userNodeForPackage( AgwpeTerminal.class );
    private static final AgwpeTerminal unproto          = new AgwpeTerminal();
    private static       String        localCall;
    private static       int           agwpePort        = 0;
    private static       String        inetHost         = "localhost";
    private static       int           inetPort         = 8000;
    private static       String        userName;
    private static       String        password;
    private static       AgwpeSocket   agwpeSocket;

           private final MouseAdapter  mouseAdapter     = new MouseAdapter(){
                        public void mousePressed( MouseEvent event ){
                            if ( event.isPopupTrigger() ){      // for Solaris
                                Tools.showMenu( jPopupMenu, event );
                            }
                        }
                        public void mouseReleased( MouseEvent event ){
                            if ( event.isPopupTrigger() ){      // for Windows
                                Tools.showMenu( jPopupMenu, event );
                            }
                        }
                    };// MouseAdapter subclass
           private final JPopupMenu    jPopupMenu       = new JPopupMenu();
           private final JTextField    messageField     = new JTextField( 100 );
           private final Logger        log              = Log.get( this.getClass() );
           private final LogPane       handler          = new LogPane( new JTextArea() );
           private final String        remoteCall;
           private       Font          textFont;

    {
        log.addHandler( handler );
        add( new JScrollPane( handler.pane ) );
        handler.pane.setLineWrap( true );
        handler.pane.setWrapStyleWord( true );
        handler.pane.addMouseListener( mouseAdapter );
        initMenus( this );
    }

    //----------------------------------------------------------------------------------------------
    /** Application main entry point.  Create the main window and call
     *  common program setup.
     *
     *  @param  args                    an array of command line arguments.
     */
    @SuppressWarnings( "fallthrough" )
    public static void main( String[] args ){
        StringTokenizer tokens;

        jFrame.getContentPane().add( jTabbedPane );
        switch ( args.length ){
        case 4:
            password = args[ 3 ];
        case 3:  // [fallthrough]
            userName = args[ 2 ];
        case 2:  // [fallthrough]
            tokens = new StringTokenizer( args[ 1 ], ":" );
            inetHost = tokens.nextToken();
            if ( tokens.hasMoreTokens() ){
                try {
                    inetPort = Integer.parseInt( tokens.nextToken() );
                } catch ( Exception exception ){
                    unproto.log.log( Level.SEVERE, "Invalid configuration TcpIp port", exception );

                }
            }
        case 1:  // [fallthrough]
            tokens = new StringTokenizer( args[ 0 ], ":" );
            localCall = tokens.nextToken().toUpperCase();
            if ( tokens.hasMoreTokens() ){
                try {
                    agwpePort = Integer.parseInt( tokens.nextToken() );
                } catch ( Exception exception ){
                    unproto.log.log( Level.SEVERE, "Invalid configuration AGWPE port", exception );
                }
            }
            break;
        case 0:
            unproto.log.severe( "*** No callsign provided ! ***" );
            break;
        }
        initMenus();
        jFrame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
        jFrame.setBounds( 200, 10, 1200, 1180 );
        jFrame.setVisible( true );
        new Thread(){
            public void run(){
                listen();
            }
        }.start();
    } // AgwpeTerminal.main

    //----------------------------------------------------------------------------------------------
    /** Initialize window menu items.
     */
    private static void initMenus(){
        JMenuBar             jMenuBar = new JMenuBar();
        JMenu                jMenu,
                             jSubMenu;
        JMenuItem            jMenuItem;
        JRadioButtonMenuItem button;
        ButtonGroup          buttonGroup;
        String               value,
                             currLnfName = UIManager.getLookAndFeel().getName();

        uiComponents.add( jFrame );
        jFrame.setJMenuBar( jMenuBar );

        jMenu = new JMenu( "File" );
        jMenu.setMnemonic( KeyEvent.VK_F );
        jMenuBar.add( jMenu );

        jMenuItem = new JMenuItem( "Connect ..." );
        jMenuItem.setMnemonic( KeyEvent.VK_C );
        jMenuItem.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    final JDialog    jDialog      = new JDialog( jFrame, "Connection Parameters",
                                                                 true );
                    final JTextField stationField = new JTextField( AgwpeSocket.Segment
                                                                               .CALLSIGN_SIZE );
                    final JTextField relayField   = new JTextField( AgwpeSocket.Segment.CALLSIGN_SIZE
                                                                  * AgwpeSocket.Segment.MAX_RELAYS );
                          JButton    jButton      = new JButton( "Connect" );
                          Box        hBox;
                          Container  container    = jDialog.getContentPane();
                    container.setLayout( new BoxLayout( container, BoxLayout.Y_AXIS ) );
                    container.add( new JPanel() );

                    hBox = new Box( BoxLayout.X_AXIS );
                    hBox.add( new JPanel() );
                    hBox.add( new JLabel( "Destination station: " ) );
                    hBox.add( stationField );
                    hBox.add( new JPanel() );
                    container.add( hBox );

                    container.add( new JPanel() );

                    hBox = new Box( BoxLayout.X_AXIS );
                    hBox.add( new JPanel() );
                    hBox.add( new JLabel( "Relay stations: " ) );
                    hBox.add( relayField );
                    hBox.add( new JPanel() );
                    container.add( hBox );

                    container.add( new JPanel() );
                    
                    hBox = new Box( BoxLayout.X_AXIS );
                    hBox.add( new JPanel() );
                    hBox.add( jButton );
                    hBox.add( new JPanel() );
                    container.add( hBox );

                    container.add( new JPanel() );
                    stationField.setMaximumSize( stationField.getPreferredSize() );

                    ActionListener connectListener = new ActionListener(){
                        public void actionPerformed( ActionEvent event ){
                            String  remoteCall = stationField.getText().toUpperCase();
                            byte[]  body = AgwpeSocket.Segment
                                            .buildRelays( relayField.getText().toUpperCase() );
                            char    type = body != null ? AgwpeSocket.Segment.Type.CONNECT_VIA
                                                        : AgwpeSocket.Segment.Type.CONNECT;
                            try {
                                agwpeSocket.send( new AgwpeSocket.Segment( agwpePort, type, 0,
                                                                   localCall, remoteCall, body ) );
                            } catch ( Exception exception ){
                                unproto.log.log( Level.SEVERE,
                                                 "Communications error sending a message",
                                                 exception );
                            }
                            jDialog.dispose();
                        }
                    }; // ActionListener implementation
                    jButton.addActionListener( connectListener );
                    stationField.addActionListener( connectListener );
                    relayField.addActionListener( connectListener );
                    jDialog.pack();
                    jDialog.setLocation( - jDialog.getWidth()  / 2,
                                         - jDialog.getHeight() / 2 );
                    jDialog.setLocationRelativeTo( jFrame );
                    jDialog.setVisible( true );
                }
            } ); // ActionListener implementation
        jMenu.add( jMenuItem );

        jMenu.addSeparator();

        jMenuItem = new JMenuItem( "Exit" );
        jMenuItem.setMnemonic( KeyEvent.VK_E );
        jMenuItem.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    switch ( JOptionPane.showConfirmDialog( jFrame, "Do you really want to exit?",
                                                            "Exit", JOptionPane.OK_CANCEL_OPTION,
                                                                    JOptionPane.QUESTION_MESSAGE ) ){
                    case JOptionPane.OK_OPTION:
                        System.exit( 0 );
                    }
                }
            } ); // ActionListener implementation
        jMenu.add( jMenuItem );

        jMenu = new JMenu( "Window" ); 
        jMenu.setMnemonic( KeyEvent.VK_W );
        jMenu.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    Component component = jTabbedPane.getSelectedComponent();
                    if ( component instanceof AgwpeTerminal ){
                        AgwpeTerminal terminal = (AgwpeTerminal) component;
                        terminal.jPopupMenu.show( component, component.getWidth()  / 2,
                                                             component.getHeight() / 2 );
                    }
                }
            } ); // ActionListener implementation
        jMenuBar.add( jMenu );

        jMenu = new JMenu( "Options" );
        jMenu.setMnemonic( KeyEvent.VK_O );
        jMenuBar.add( jMenu );

        // Set up Look & Feel submenu
        value = preferences.get( "Window.lookAndFeel", "Native" );
        jSubMenu = new JMenu( "Look & Feel" );
        buttonGroup = new ButtonGroup();
        jMenuItem = null;
        for ( UIManager.LookAndFeelInfo lookAndFeelInfo : Tools.lookAndFeelInfos ){
            final String    className   = lookAndFeelInfo.getClassName();
            final String    name        = lookAndFeelInfo.getName();
                  int       key         = Character.toUpperCase( name.charAt( 0 ) );
            button = new JRadioButtonMenuItem( name, name.equals( currLnfName ) );
            button.setToolTipText( "Set the Look & Feel to " + name );
            button.addActionListener( new ActionListener(){
                    public void actionPerformed( ActionEvent event ){
                        try {
                            UIManager.setLookAndFeel( className );
                            unproto.log.config( "Setting: "
                                              + UIManager.getLookAndFeel().getDescription() );

                            for ( Component component : uiComponents ) {
                                SwingUtilities.updateComponentTreeUI( component );
                            }
                            preferences.put( "Window.lookAndFeel", name );
                        } catch ( Exception exception ){  // multiple
                            unproto.log.log( Level.WARNING, "Error setting the Look & Feel",
                                             exception );
                        }
                    }
                } ); // ActionListener implementation
            buttonGroup.add( button );
            jSubMenu.add( button );

            if ( name.equalsIgnoreCase( value )
                    || "Native".equalsIgnoreCase( value )
                        &&  className.equals( UIManager.getSystemLookAndFeelClassName() ) ){
                jMenuItem = button;
            }

        }
        jMenu.add( jSubMenu );
        if ( jMenuItem != null ){
            jMenuItem.doClick();
        } else {
            unproto.log.warning( "*** '" + value
                               + "' initial look and feel not found, using default ***" );
        }

        jMenu = new JMenu( "Help" );
        jMenu.setMnemonic( KeyEvent.VK_H );
        jMenuBar.add( jMenu );

        jMenuItem = new JMenuItem( "About" );
        jMenuItem.setMnemonic( KeyEvent.VK_A );
        jMenuItem.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    JOptionPane.showMessageDialog( jFrame,  TITLE + Log.endOfLine + COPYRIGHT,
                                                   "About", JOptionPane.INFORMATION_MESSAGE );
                }
            } ); // ActionListener implementation
        jMenu.add( jMenuItem );
    } // AgwpeTerminal.initMenus

    //----------------------------------------------------------------------------------------------
    /** Initialize window menu items.
     *
     *  @param  terminal                the terminal window Container.
     */
    private void initMenus( final AgwpeTerminal terminal ){
        JMenu                jSubMenu;
        JMenuItem            jMenuItem;
        JRadioButtonMenuItem button;
        ButtonGroup          buttonGroup;
        String               value;

        uiComponents.add( jPopupMenu );

        jMenuItem = new JMenuItem( "Disconnect" );
        jMenuItem.setMnemonic( KeyEvent.VK_D );
        jMenuItem.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    Component component = jTabbedPane.getSelectedComponent();
                    if ( terminal.remoteCall != null ){
                        try {
                            agwpeSocket.send( new AgwpeSocket
                                                  .Segment( agwpePort,
                                                            AgwpeSocket.Segment.Type.DISCONNECT, 0,
                                                            localCall, terminal.remoteCall, null ) );
                        } catch ( Exception exception ){
                            unproto.log.log( Level.SEVERE, "Communications error sending a message",
                                             exception );
                        }
                        return;
                    }
                    JOptionPane
                     .showMessageDialog( jFrame, "This window does not have an active connection !",
                                                 "Disconnect", JOptionPane.ERROR_MESSAGE );
                }
            } ); // ActionListener implementation
        jPopupMenu.add( jMenuItem );

        jMenuItem = new JMenuItem( "Clear window" );
        jMenuItem.setMnemonic( KeyEvent.VK_C );
        jMenuItem.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    terminal.handler.clear();
                }
            } ); // ActionListener implementation
        jPopupMenu.add( jMenuItem );

        jMenuItem = new JMenuItem( "Delete window" );
        jMenuItem.setMnemonic( KeyEvent.VK_D );
        jMenuItem.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    String    text      = "Non-connection windows cannot be deleted !";
                    Component component = jTabbedPane.getSelectedComponent();
                    if ( terminal.remoteCall != null ){
                        text = "Windows with active connections cannot be deleted !";
                        if ( ! terminal.messageField.isEditable() ){
                            jTabbedPane.remove( component );
                            return;
                        }
                    }
                    JOptionPane.showMessageDialog( jFrame, text, "Tabbed Window Delete",
                                                   JOptionPane.ERROR_MESSAGE );
                }
            } ); // ActionListener implementation
        jPopupMenu.add( jMenuItem );

        // Set up Font menu item
        jMenuItem = new JMenuItem( "Select Font" );
        jMenuItem.setToolTipText( "Select the default text font" );
        jMenuItem.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    Component component = jTabbedPane.getSelectedComponent();
                    JFontChooser jFontChooser = new JFontChooser( jFrame, "Select a font",
                                                                  terminal.textFont );
                    jFontChooser.setVisible( true );
                    Font font = jFontChooser.getSelectedFont();
                    if ( font != null ){
                        terminal.textFont = font;
                        terminal.handler.pane.setFont( font );
                        if ( terminal == unproto ){
                            preferences.put(    "Window.font.name",  font.getName() );
                            preferences.putInt( "Window.font.style", font.getStyle() );
                            preferences.putInt( "Window.font.size",  font.getSize() );
                        }
                        terminal.log.config( "Setting: font '" + font.getName() + "', "
                                                               + font.getSize() + "pt" );
                    }
                }
            } ); // ActionListener implementation
        jPopupMenu.add( jMenuItem );

        // Set up Log Options submenu
        String currentName = (unproto != null ? unproto : terminal).handler.getLevel().getName();
        value = preferences.get( "Log.level", currentName );
        jSubMenu = new JMenu( "Logging level" );
        buttonGroup = new ButtonGroup();
        jMenuItem = null;
        for ( final Level level : LogPane.levels ){
            final String name = level.getName();
            button = new JRadioButtonMenuItem( Tools.initialCase( name ),
                                               name.equals( currentName ) );
            button.setToolTipText( "Set the " + name + " level of activity logging" );
            button.addActionListener( new ActionListener(){
                    public void actionPerformed( ActionEvent event ){
                        Component component = jTabbedPane.getSelectedComponent();
                        if ( component instanceof AgwpeTerminal ){
                            AgwpeTerminal terminal = (AgwpeTerminal) component;
                            terminal.handler.setLevel( level );
                            if ( terminal == unproto ){
                                preferences.put( "Log.level", name );
                            }
                            terminal.log.config( "Setting: The " + name
                                               + " level of activity logging" );
                        }
                    }
                } ); // ActionListener implementation
//          keyStrokeMap.put( KeyStroke.getKeyStroke( logLevel.charAt( 0 ) ), button );
            buttonGroup.add( button );
            jSubMenu.add( button );
            if ( name.equalsIgnoreCase( value ) ){
                jMenuItem = button;
            }
        }
        jPopupMenu.add( jSubMenu );
        if ( jMenuItem != null ){
            jMenuItem.doClick();
        } else {
            unproto.log.warning( "'" + value + "' initial logging level not found, using default" );
        }
    } // AgwpeTerminal.initMenus

    //----------------------------------------------------------------------------------------------
    /** Listen for AGWPE message segments and dispatch them.
     */
    private static void listen(){
        AgwpeTerminal           terminal;
        AgwpeSocket.Segment     agwpeSocketSegment;
        String                  callsign;

        try {
            agwpeSocket = new AgwpeSocket( inetHost, inetPort );
            agwpeSocket.send( new AgwpeSocket.Segment( agwpePort, AgwpeSocket.Segment.Type.VERSION,
                                                       0, null, null, null ) );
            agwpeSocketSegment = agwpeSocket.receive();
            unproto.log.info( "AGWPE version: "
                            + AgwpeSocket.Segment.extract (agwpeSocketSegment.body, 0 ) + "."
                            + AgwpeSocket.Segment.extract (agwpeSocketSegment.body, 4 ) );
            if ( userName != null ){
                byte[] body = new byte[ 510 ];
                AgwpeSocket.Segment.insert( userName, body, 0 );
                AgwpeSocket.Segment.insert( password, body, 255 );
                agwpeSocket.send( new AgwpeSocket.Segment( agwpePort, AgwpeSocket.Segment.Type.LOGIN,
                                                           0, null, null, body ) );
            }
            agwpeSocket.send( new AgwpeSocket.Segment( agwpePort, AgwpeSocket.Segment.Type.REGISTER,
                                                       0, localCall, null, null ) );
            agwpeSocketSegment = agwpeSocket.receive();
            unproto.log.info( localCall + " registration "
                            + (agwpeSocketSegment.body[ 0 ] != 0  ? "succeeded" : "failed") );
            agwpeSocket.send( new AgwpeSocket.Segment( agwpePort,
                                AgwpeSocket.Segment.Type.PORTS_INFO,     0, null, null, null ) );
            agwpeSocket.send( new AgwpeSocket.Segment( agwpePort,
                                AgwpeSocket.Segment.Type.MONITOR_TOGGLE, 0, null, null, null ) );
            agwpeSocket.send( new AgwpeSocket.Segment( agwpePort,
                                AgwpeSocket.Segment.Type.HEARD,          0, null, null, null ) );
            while ( true ){
                agwpeSocketSegment = agwpeSocket.receive();
                unproto.log.finer( "===== Port: " + agwpeSocketSegment.port
                                    +  "; type: " + agwpeSocketSegment.type
                                    +   "; pid: " + agwpeSocketSegment.pid
                                    +  "; from: " + agwpeSocketSegment.from
                                    +    "; to: " + agwpeSocketSegment.to + " =====" );
                switch ( agwpeSocketSegment.type ){
                case AgwpeSocket.Segment.Type.PORTS_INFO:
                    break;
                case AgwpeSocket.Segment.Type.HEARD:
                    if ( agwpeSocketSegment.from.trim().length() == 0 ){
                        continue;
                    }
                    break;
                case AgwpeSocket.Segment.Type.CONNECT:
                    terminal = new AgwpeTerminal( agwpeSocketSegment.from );
                    terminal.log.info( agwpeSocketSegment.body() );
                    hashMap.put( agwpeSocketSegment.from, terminal );
                    continue;
                case AgwpeSocket.Segment.Type.DISCONNECT:
                    terminal = hashMap.get( agwpeSocketSegment.from );
                    if ( terminal != null ){
                        terminal.messageField.setEditable( false );
                        terminal.log.info( agwpeSocketSegment.body() );
                        terminal.handler.pane.setBackground( Color.LIGHT_GRAY );
                        jTabbedPane.setTitleAt( jTabbedPane.indexOfComponent( terminal ),
                                                "[" + terminal.remoteCall + "]" );
                        hashMap.remove( terminal.remoteCall );
                        continue;
                    }
                    break;
                case AgwpeSocket.Segment.Type.CONNECTED_DATA:
                    terminal = hashMap.get( agwpeSocketSegment.from );
                    if ( terminal != null ){
                        terminal.log.log( Level.ALL, "[" + agwpeSocketSegment.type + "] "
                                                         + agwpeSocketSegment.body() );
                        continue;
                    }
                    break;
                case AgwpeSocket.Segment.Type.MONITORED_FRAME:
                    terminal = hashMap.get( agwpeSocketSegment.to );
                    if ( terminal != null ){
                        terminal.log.finest( "[" + agwpeSocketSegment.type + "] "
                                                 + agwpeSocketSegment.body() );
                        continue;
                    }
                    break;
                case AgwpeSocket.Segment.Type.INFO_TRAME:
                case AgwpeSocket.Segment.Type.SUPERVISORY_FRAME:
                case AgwpeSocket.Segment.Type.RAW_FRAME:
                    callsign = localCall.equals( agwpeSocketSegment.to )
                                ? agwpeSocketSegment.from : agwpeSocketSegment.to;
                    terminal = hashMap.get( callsign );
                    if ( terminal != null ){
                        terminal.log.finest( "[" + agwpeSocketSegment.type + "] "
                                                 + agwpeSocketSegment.body() );
                        continue;
                    }
                    break;
                case AgwpeSocket.Segment.Type.UNCONNECTED_FRAME:
                    break;
                }
                unproto.log.finest( "[" + agwpeSocketSegment.type + "] "
                                        + agwpeSocketSegment.body() );
            }
        } catch ( Exception exception ){
            unproto.log.log( Level.SEVERE, "Communications error", exception );
        }
    } // AgwpeTerminal.listen

    //----------------------------------------------------------------------------------------------
    /** Initialize the default window tab.
     */
    private AgwpeTerminal(){
        super( BoxLayout.Y_AXIS );
        final   JTextField      stationField    = new JTextField( "CQ",
                                                                  AgwpeSocket.Segment.CALLSIGN_SIZE );
        final   JTextField      relayField      = new JTextField( AgwpeSocket.Segment.MAX_RELAYS
                                                                 * AgwpeSocket.Segment.CALLSIGN_SIZE );
                JButton         clearButton     = new JButton( "Clear" );
                Box             vBox            = new Box( BoxLayout.Y_AXIS );
                Box             hBox;

        add( vBox );
        jTabbedPane.addTab( "Unconnected", null, this, "Unconnected activity" );
        handler.setLevel( Level.CONFIG );
        this.remoteCall = null;

        vBox.add( new JPanel() );

        hBox = new Box( BoxLayout.X_AXIS );
        hBox.add( new JPanel() );
        hBox.add( new JLabel( "Destination station: " ) );
        hBox.add( stationField );
        hBox.add( new JPanel() );
        hBox.add( new JLabel( "Relay stations: " ) );
        hBox.add( relayField );
        hBox.add( new JPanel() );
        vBox.add( hBox );

        vBox.add( new JPanel() );

        hBox = new Box( BoxLayout.X_AXIS );
        hBox.add( new JPanel() );
        hBox.add( new JLabel( "Message text: " ) );
        hBox.add( messageField );
        hBox.add( new JPanel() );
        hBox.add( clearButton );
        hBox.add( new JPanel() );
        vBox.add( hBox );

        vBox.add( new JPanel() );
        vBox.setMaximumSize( vBox.getPreferredSize() );

        clearButton.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    messageField.setText( null );
                }
            } ); // ActionListener implementation
        ActionListener transmitListener = new ActionListener(){
            public void actionPerformed( ActionEvent event ){
                String   remoteCall = stationField.getText().toUpperCase();
                String   text       = event.getActionCommand() + endOfLine;
                byte[]   body       = AgwpeSocket.Segment
                                       .buildRelays( relayField.getText().toUpperCase() );
                char     type       = body != null ? AgwpeSocket.Segment.Type.TRANSMIT_VIA
                                                   : AgwpeSocket.Segment.Type.TRANSMIT;
                if ( body != null ){
                    text = new String( body ) + text;
                }
                try {
                    agwpeSocket.send( new AgwpeSocket.Segment(  agwpePort, type, 0, localCall,
                                                                remoteCall, text.getBytes() ) );
                } catch ( Exception exception ){
                    log.log( Level.SEVERE, "Communications error sending a message", exception );
                }
            }  
        }; // ActionListener implementation
        messageField.addActionListener( transmitListener );
        textFont = new Font( preferences.get(    "Window.font.name", "Monospaced" ),
                             preferences.getInt( "Window.font.style", Font.PLAIN ),
                             preferences.getInt( "Window.font.size", 14 ) );
        handler.pane.setFont( textFont );
    } // AgwpeTerminal constructor

    //----------------------------------------------------------------------------------------------
    /** Initialize a connected window tab.
     *
     *  @param  remoteCall              the remote station's callsign and SSID.
     */
    private AgwpeTerminal( final String remoteCall ){
        super( BoxLayout.Y_AXIS );
        JButton clearButton     = new JButton( "Clear" );
        Box     vBox            = new Box( BoxLayout.Y_AXIS );
        Box     hBox;

        add( vBox );
        jTabbedPane.insertTab( remoteCall, null, this, remoteCall + " activity", 0 );
        handler.setLevel( Level.INFO );
        this.remoteCall = remoteCall;

        vBox.add( new JPanel() );

        hBox = new Box( BoxLayout.X_AXIS );
        hBox.add( new JPanel() );
        hBox.add( new JLabel( "Message text:" ) );
        hBox.add( messageField );
        hBox.add( new JPanel() );
        hBox.add( clearButton );
        hBox.add( new JPanel() );
        vBox.add( hBox );

        vBox.add( new JPanel() );
        vBox.setMaximumSize( vBox.getPreferredSize() );

        clearButton.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    messageField.setText( null );
                }
            } ); // ActionListener implementation
        ActionListener transmitListener = new ActionListener(){
            public void actionPerformed( ActionEvent event ){
                char type = AgwpeSocket.Segment.Type.CONNECTED_DATA;
                String text = event.getActionCommand() + endOfLine;
                try {
                    agwpeSocket.send( new AgwpeSocket.Segment( agwpePort, type, 0, localCall,
                                                            remoteCall, text.getBytes() ) );
                } catch ( Exception exception ){
                    log.log( Level.SEVERE, "Communications error sending a message", exception );
                }
            }  
        }; // ActionListener implementation
        messageField.addActionListener( transmitListener );
        textFont = unproto.textFont;
        handler.pane.setFont( textFont );
        repaint();
//      revalidate();
    } // AgwpeTerminal constructor
} // class AgwpeTerminal
