package com.ultimeth.util;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

//==================================================================================================
/** JFontChooser provides a font selection dialog box.
 *
 *  @version    2005.08.12
 *  @author     Copyright © 2002 by Dean K. Gibson.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
@SuppressWarnings( "serial" )
public class JFontChooser extends JDialog {
    private static final DefaultListModel familyListModel  = new DefaultListModel();

           private       Font   result  = null,
                         font;
           private       float  fontSize;

    static {
        TreeMap <String, Pair <String, DefaultListModel>> treeMap
            = new TreeMap <String, Pair <String, DefaultListModel>>();
        for ( Font font: GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts() ){
            String                          familyName  = font.getFamily(),
                                            fontName    = font.getName();
            Pair <String, DefaultListModel> pair        = treeMap.get( familyName.toLowerCase() );
            if ( pair == null ){
                pair = new Pair <String, DefaultListModel>( familyName, new DefaultListModel() );
                treeMap.put( familyName.toLowerCase(), pair );
            }
            pair.value.addElement( new Pair <String, Font>( fontName, font ) );
        }
        for ( Pair <String, DefaultListModel> pair : treeMap.values() ){
            familyListModel.addElement( pair );
        }
    }

    //----------------------------------------------------------------------------------------------
    /** FontDialog constructor.
     *
     *  @param  window                  the Container window.
     *  @param  title                   the JDialog box title.
     *  @param  initialFont             the initial Font.
     */
    @SuppressWarnings( "unchecked" )
    public JFontChooser( Container window, String title, Font initialFont ){
        super( window instanceof Frame ? (Frame) window : null, title, true );
                Box             main            = Box.createVerticalBox(),
                                vBoxL           = Box.createVerticalBox(),
                                vBoxR           = Box.createVerticalBox(),
                                hBox;                           // reused!
                JButton         jButton;                        // reused!
        final   JDialog         jDialog         = this;
        final   JSlider         jSlider;;
        final   JTextComponent  jTextComponent  = new JTextArea( "ABC abc" );
        final   JList           familyList      = new JList( familyListModel ),
                                fontList        = new JList();

        fontSize = initialFont != null ? initialFont.getSize() : 14;

        familyList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        familyList.addListSelectionListener( new ListSelectionListener(){
                public void valueChanged( ListSelectionEvent event ){
                    Object object = familyList.getSelectedValue();
                    if ( object instanceof Pair ){
                        fontList.setModel( (ListModel) ((Pair) object).value );
                        fontList.setSelectedIndex( 0 );
                    }
                } 
            } ); // ListSelectionListener implementation

        fontList.setSelectionMode(   ListSelectionModel.SINGLE_SELECTION );
        fontList.addListSelectionListener( new ListSelectionListener(){
                public void valueChanged( ListSelectionEvent event ){
                    Object object = fontList.getSelectedValue();
                    if ( object instanceof Pair ){
                        font = ((Font) ((Pair) object).value).deriveFont( fontSize );
                        jTextComponent.setFont( font );
                    }
                } 
            } ); // ListSelectionListener implementation

        jSlider = new JSlider( 10, 72, (int) fontSize );
        jSlider.setMinorTickSpacing( 1 );
        jSlider.setMajorTickSpacing( 5 );
        jSlider.setPaintTicks( true );
        jSlider.setPaintLabels( true );
        jSlider.addChangeListener( new ChangeListener(){
                public void stateChanged( ChangeEvent event ) {
                    fontSize = jSlider.getValue();
                    font = font.deriveFont( fontSize );
                    jTextComponent.setFont( font );
                }
            } ); // ChangeListener implementation

        jTextComponent.setPreferredSize( new Dimension( 400, 100 ) );

        getContentPane().add( main );

        hBox = Box.createHorizontalBox();
        main.add( hBox );
        hBox.add( vBoxL );
        hBox.add( vBoxR );

        hBox = Box.createHorizontalBox();
        hBox.add( new JLabel( "Font families" ) );
        hBox.add( Box.createHorizontalGlue() );
        vBoxL.add( hBox );
        vBoxL.add( new JScrollPane( familyList ) );

        hBox = Box.createHorizontalBox();
        hBox.add( new JLabel( "Font faces" ) );
        hBox.add( Box.createHorizontalGlue() );
        vBoxR.add( hBox );

        vBoxR.add( new JScrollPane( fontList ) );

        hBox = Box.createHorizontalBox();
        hBox.add( new JLabel( "Font size" ) );
        hBox.add( Box.createHorizontalGlue() );
        vBoxR.add( hBox );

        vBoxR.add( jSlider );

        hBox = Box.createHorizontalBox();
        hBox.add( new JLabel( "Sample typing area" ) );
        hBox.add( Box.createHorizontalGlue() );
        vBoxR.add( hBox );

        vBoxR.add( jTextComponent );

        hBox = Box.createHorizontalBox();
        main.add( hBox );

        jButton = new JButton( "Set" );
        jButton.setToolTipText( "Apply changes" );
        jButton.addActionListener(
            new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    result = font;
                    jDialog.dispose();
                }
            } ); // ActionListener implementation
        hBox.add( jButton );

        jButton = new JButton( "Cancel" );
        jButton.setToolTipText( "Ignore any changes" );
        jButton.addActionListener(
            new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    jDialog.dispose();
                }
            } ); // ActionListener implementation
        hBox.add( jButton );

        String match = initialFont != null ? initialFont.getFamily() : "";
        int    index = 0;
        for ( Object familyPair : new ToIterable( familyListModel.elements() ) ){  // [unchecked]
            if ( familyPair.toString().equals( match ) ){
                DefaultListModel nameListModel = ((Pair <String, DefaultListModel>) familyPair).value;  // [unchecked]
                familyList.setSelectedIndex(     index );
                familyList.ensureIndexIsVisible( index );
                match = initialFont != null ? initialFont.getName() : "";
                index = 0;
                for ( Object namePair : new ToIterable( nameListModel.elements() ) ){  // [unchecked]
                    if ( namePair.toString().equals( match ) ){
                        fontList.setSelectedIndex(     index );
                        fontList.ensureIndexIsVisible( index );
                        break;
                    }
                    index++;
                }
                break;
            }
            index++;
        }

        pack();
        setLocation( -getWidth() / 2, -getHeight() / 2 );
        setLocationRelativeTo( window );
    }// JFontChooser constructor

    //----------------------------------------------------------------------------------------------
    /** Gets the selected Font.
     *
     *  @return Font            - the selected Font.
     */
    public Font getSelectedFont(){
        return result;
    }// JFontChooser.getSelectedFont
}// JFontChooser class
