package com.ultimeth.util;

import java.util.*;
import java.util.logging.*;

//==================================================================================================
/** A logging capability using a StringBuffer.
 *
 *  @version    2005.08.25
 *  @author     Copyright © 2002-2005 by Dean K. Gibson.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class LogBuff extends Log.Handler {

    private StringBuffer buffer;
    private int          length;

    //----------------------------------------------------------------------------------------------
    /** Initializes a Logger.
     *
     *  @param  length                  the initial size of the StringBuffer.
     */
    public LogBuff( int length ){
        this.length  = length;
        clear();
    }// LogBuff constructor

    //----------------------------------------------------------------------------------------------
    /** Publishes a LogRecord (Handler override).
     *
     *  @param  logRecord               the LogRecord to publish.
     */
    public void publish( LogRecord logRecord ){
        if ( isLoggable( logRecord ) ){
            synchronized ( buffer ){
                    buffer.append( getFormatter().format( logRecord ) );  // thread safe!
            }
        }
    }// LogBuff.publish

    //----------------------------------------------------------------------------------------------
    /** Clears the buffer.
     */
    public void clear(){
        buffer = new StringBuffer( length );
    }// LogBuff.clear

    //----------------------------------------------------------------------------------------------
    /** Gets the buffer contents.
     *
     *  @return String                  - the contents of the buffer.
     */
    public String get(){
        return buffer.toString();
    }// LogBuff.get
}// LogBuff class
