package com.ultimeth.util;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.logging.*;
import javax.swing.*;

//==================================================================================================
/** A logging capability using a JTextArea.
 *
 *  @version    2005.08.25
 *  @author     Copyright © 2002-2005 by Dean K. Gibson.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class LogPane extends Log.Handler {
    public static final Level[] levels = { Level.SEVERE,
                                           Level.WARNING,
                                           Level.INFO,
                                           Level.CONFIG,
                                           Level.FINE,
                                           Level.ALL };

    public  final JTextArea pane;

    //----------------------------------------------------------------------------------------------
    /** Initializes a Logger.
     *
     *  @param  pane                    the JTextArea to log to.
     */
    public LogPane( JTextArea pane ){
        this.pane = pane;
        pane.setEditable( false );
    }// LogPane constructor

    //----------------------------------------------------------------------------------------------
    /** Publishes a LogRecord (Handler override).
     *
     *  @param  logRecord               the LogRecord to publish.
     */
    public void publish( LogRecord logRecord ){
        if ( isLoggable( logRecord ) ){
            pane.append( getFormatter().format( logRecord ) );  // thread safe!
            pane.setCaretPosition( pane.getDocument().getLength() );
        }
    }// LogPane.publish

    //----------------------------------------------------------------------------------------------
    /** Clears the JTextArea.
     */
    public void clear(){
        pane.setText( null );           // thread safe!
    }// LogPane.clear
}// LogPane class
